package com.yixin.ms.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yixin.ms.boot.uitls.DateUtils;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.model.dto.IrsShortTermDTO;
import com.yixin.ms.model.dto.IrsTripCheckScheduleDTO;
import com.yixin.ms.model.dto.IrsTripSendMsgDTO;
import com.yixin.ms.model.enums.IrsTripAuthEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.IrsTripServer;
import com.yixin.ms.server.vo.IrsTripResultVO;
import com.yixin.ms.server.vo.IrsTripTokenVO;
import com.yixin.ms.service.IrsTripService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author: lileilei
 * @date: 2022/8/5 13:48
 * @description: 省共享工信行程
 */
@Slf4j
@Service
public class IrsTripServiceImpl implements IrsTripService {
    @Autowired
    private IrsTripServer irsTripServer;

    @Override
    public String sendMsg(IrsTripSendMsgDTO sendMsgDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(sendMsgDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(sendMsgDTO.getAppSecret())) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.FAIL_TOKEN_FLAG).setMsg("调用权限校验失败，请检查"));
        }

        IrsTripTokenVO irsTripTokenVO = irsTripServer.getToken();
        log.info("请求token返回内容: {}", JSON.toJSONString(irsTripTokenVO));
        return irsTripServer.sendMsg(sendMsgDTO.getPhone(), irsTripTokenVO.getRequestSecret());
    }

    @Override
    public String checkSchedule(IrsTripCheckScheduleDTO checkScheduleDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(checkScheduleDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(checkScheduleDTO.getAppSecret())) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.FAIL_TOKEN_FLAG).setMsg("调用权限校验失败，请检查"));
        }

        IrsTripTokenVO irsTripTokenVO = irsTripServer.getToken();
        return irsTripServer.checkSchedule(checkScheduleDTO.getPhone(), checkScheduleDTO.getVerification(), irsTripTokenVO.getRequestSecret(), checkScheduleDTO.getCity());
    }

    @Override
    public String getProvinceHealthCode(IrsShortTermDTO irsShortTermDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(irsShortTermDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(irsShortTermDTO.getAppSecret())) {
            return JSON.toJSONString(new IrsTripResultVO<>().setCode(IrsTripResultVO.FAIL_TOKEN_FLAG).setMsg("调用权限校验失败，请检查"));
        }
        HealtCodeUserVO userFromProvince = getShortTermUserFromProvince(irsShortTermDTO.getIdNumber(), null);
        log.info("--->getProvinceHealthCode省健康码最终返回数据：{}", Preconditions.isNotBlank(userFromProvince) ? JSONObject.toJSONString(userFromProvince) : null);
        return Preconditions.isNotBlank(userFromProvince) ? JSONObject.toJSONString(userFromProvince) : null;
    }

    /**
     * 查询省健康码基本信息、疫苗注射数据信息、核酸检测数据 自定义封装三合一接口
     *
     * @param idNumber 身份证号
     * @param name     姓名
     * @return
     */
    private HealtCodeUserVO getShortTermUserFromProvince(String idNumber, String name) {

        try {
            HealtCodeUserVO shortTermUser = new HealtCodeUserVO();
            //查询省健康码基本信息
            getHealthCodeBaseInfo(shortTermUser, idNumber, name);

            //疫苗注射数据信息
            getVaccineNum(shortTermUser, idNumber, name);

            //核酸检测数据列表
            getNucleinList(shortTermUser, idNumber, name);

            //校验数据完整性
            if (checkDataIntegrity(shortTermUser)) {
                return shortTermUser;
            }
        } catch (Exception e) {
            log.error("数据异常",e);
            //TODO。。。
        }
        return null;
    }


    /**
     * 校验数据完整性
     *
     * @param shortTermUser
     * @return
     */
    private boolean checkDataIntegrity(HealtCodeUserVO shortTermUser) {
        if (Preconditions.isNotBlank(shortTermUser)) {
            int count = 0;
            if (Preconditions.isNotBlank(shortTermUser.getNuclein())
                    && shortTermUser.getNuclein().size() > 0) {
                count++;
            }

            if (Preconditions.isNotBlank(shortTermUser.getData())
                    && shortTermUser.getData().size() > 0) {
                count++;
            }

            if (Preconditions.isNotBlank(shortTermUser.getVaccineNum())) {
                count++;
            }

            if (count > 1) {
                return true;
            }
        }
        return false;
    }

    /**
     * 核酸检测数据列表
     *
     * @param shortTermUser 数据封装
     * @param idNumber      身份证号
     * @param name          姓名
     */
    private void getNucleinList(HealtCodeUserVO shortTermUser, String idNumber, String name) {
        String body = irsTripServer.getNucleinList(idNumber, name);
        if (StringUtils.isNotBlank(body)) {
            log.info("--->核酸检测数据列表返回原始数据：{}", JSONObject.toJSONString(body));
            JSONObject json = JSON.parseObject(body);
            if (StringUtils.equals("00", json.getString("code"))) {
                if (Preconditions.isNotBlank(json.get("datas"))
                        && Preconditions.isNotBlank(json.getJSONObject("datas").get("data"))) {
                    HealtCodeNucleinVO vo = new HealtCodeNucleinVO();
                    JSONObject object = json.getJSONObject("datas").getJSONObject("data");
                    if(Preconditions.isNotBlank(object.get("collectTime"))) {
                        vo.setChecktime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_T_HH_MM_SS, object.getString("collectTime")));
                    }
                    if(Preconditions.isNotBlank(object.get("checktime"))) {
                        vo.setReportTime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_T_HH_MM_SS, object.getString("checktime")));
                    }
                    if (Preconditions.isNotBlank(object.get("result"))) {
                        vo.setResult(object.getString("result"));
                    }
                    shortTermUser.setNuclein(Arrays.asList(vo));
                }
            }
        }
    }

    /**
     * 疫苗注射数据信息
     *
     * @param shortTermUser 数据封装
     * @param idNumber      身份证号
     * @param name          姓名
     */
    private void getVaccineNum(HealtCodeUserVO shortTermUser, String idNumber, String name) {
        String body = irsTripServer.getVaccine(idNumber, name);
        if (StringUtils.isNotBlank(body)) {
            log.info("--->疫苗注射数据信息返回原始数据：{}", JSONObject.toJSONString(body));
            JSONObject json = JSON.parseObject(body);
            if (StringUtils.equals("00", json.getString("code"))) {
                if (Preconditions.isNotBlank(json.get("datas"))
                        && Preconditions.isNotBlank(json.getJSONObject("datas").get("data"))) {
                    shortTermUser.setVaccineNum(String.valueOf(json.getJSONObject("datas").getJSONArray("data").size()));
                }
            }
        }

    }

    /**
     * 查询省健康码基本信息
     *
     * @param shortTermUser 数据封装
     * @param idNumber      身份证号
     * @param name          姓名
     */
    private void getHealthCodeBaseInfo(HealtCodeUserVO shortTermUser, String idNumber, String name) {
        String body = irsTripServer.getHealthCodeBaseInfo(idNumber, name);
        if (StringUtils.isNotBlank(body)) {
            log.info("--->查询省健康码基本信息返回原始数据：{}", JSONObject.toJSONString(body));
            JSONObject json = JSON.parseObject(body);
            if (StringUtils.equals("00", json.getString("code"))) {
                if (Preconditions.isNotBlank(json.get("datas"))) {
                    JSONArray data = json.getJSONObject("datas").getJSONArray("data");
                    List<HealtCodeTypeVO> vos = new ArrayList<>();
                    for (Object item : data) {
                        HealtCodeTypeVO vo = new HealtCodeTypeVO();
                        JSONObject jsonObject = JSONObject.parseObject(item.toString());
                        if(Preconditions.isNotBlank(jsonObject.get("hmcmyy"))) {
                            vo.setContent(jsonObject.getString("hmcmyy"));
                        }

                        if(Preconditions.isNotBlank(jsonObject.get("mzt"))) {
                            vo.setMzt(jsonObject.getString("mzt"));
                        }

                        if(Preconditions.isNotBlank(jsonObject.get("sfzh"))) {
                            vo.setIdNumber(jsonObject.getString("sfzh"));
                            shortTermUser.setIdNumber(jsonObject.getString("sfzh"));
                        }

                        if(Preconditions.isNotBlank(jsonObject.get("xm"))) {
                            vo.setUsername(jsonObject.getString("xm"));
                            shortTermUser.setUsername(jsonObject.getString("xm"));
                        }

                        if(Preconditions.isNotBlank(jsonObject.get("sjh"))) {
                            vo.setPhone(jsonObject.getString("sjh"));
                            shortTermUser.setPhone(jsonObject.getString("sjh"));
                        }
                        vos.add(vo);
                    }
                    shortTermUser.setData(vos);
                }
            }
        }
    }

}
